@@ -99,6 +99,9 @@ else {
 my @follow = ();
 # Which types of dependencies do we show?
 my @show = ();
+# store of all ProvidesList because this is indexed under what is
+# provided, not what provides.
+my %provides=();
 
 # We don't print package states by default.
 my $printstate = 0;
@@ -137,7 +140,7 @@ if A is later.
 
 =cut
 
-sub ver_compare($$)
+sub ver_compare
 {
 	my $vers = $_system->versioning;
 	my $ver_a = shift;
@@ -162,7 +165,7 @@ whichever cache was initialised most recently.
 
 =cut
 
-sub init_host_cache($)
+sub init_host_cache
 {
 	$verbose = shift;
 	return ($host_config) if (defined($host_config));
@@ -191,7 +194,7 @@ the call returns.
 
 =cut
 
-sub init_cache($)
+sub init_cache
 {
 	$verbose = shift;
 	return ($config) if (defined($config));
@@ -216,7 +219,7 @@ raw apt-pkg-perl data or undef on error.
 
 =cut
 
-sub srclookup($) {
+sub srclookup {
 	my $name = shift;
 	my $res = $apt_source->{$name};
 
@@ -230,7 +233,7 @@ raw apt-pkg-perl data or undef on error.
 
 =cut
 
-sub binlookup($) {
+sub binlookup {
 	my $name = shift;
 	my $res = $apt_cache->packages->lookup($name);
 
@@ -246,7 +249,7 @@ suitable to be dropped into the AptCrossPackage::Depends variable.
 
 =cut
 
-sub deplookup($) {
+sub deplookup {
 	@depends=();
 	my $pkg = shift;
 	&show_rdepends($pkg);
@@ -283,7 +286,7 @@ Example:
 
 =cut
 
-sub get_cache_iter() {
+sub get_cache_iter {
 	my $iter = AptPkg::Cache::Iter->new($apt_cache);
 	return $iter || undef;
 }
@@ -296,13 +299,13 @@ sources.list files that your script will later specify to Config.
 
 =cut
 
-sub set_cachedir ($)
+sub set_cachedir
 {
 	$cachedir = shift;
 	if (not -d $cachedir)
 	{
 		undef $cachedir;
-		carp ("Unable to use $cachedir: directory must exist.\n");
+		carp ("Unable to use requested cache directory: does not exist.\n");
 		return;
 	}
 	open (TEST, ">$cachedir/lock") or croak ("unable to use $cachedir: $!");
@@ -322,7 +325,7 @@ If cachedir has been set and is usable, returns $cachedir.
 
 =cut
 
-sub get_aptcross_dir ()
+sub get_aptcross_dir
 {
 	if (defined($cachedir))
 	{
@@ -345,7 +348,7 @@ so use this function to get the location of the cache.
 
 =cut
 
-sub get_cachedir ()
+sub get_cachedir
 {
 	return $cachedir if (defined($cachedir));
 	return &get_aptcross_dir();
@@ -357,7 +360,7 @@ Return the current verbosity setting.
 
 =cut
 
-sub get_verbose() {
+sub get_verbose {
 	return $verbose;
 }
 
@@ -374,7 +377,7 @@ to be used to create debugging output to the console.
 
 =cut
 
-sub set_verbose($) {
+sub set_verbose {
 	$verbose = shift;
 }
 
@@ -386,7 +389,7 @@ to load the cache data from the mirror.
 
 =cut
 
-sub set_mirror ($) {
+sub set_mirror {
 	$mirror = shift;
 }
 
@@ -398,7 +401,7 @@ command.
 
 =cut
 
-sub force_update()
+sub force_update
 {
 	$apt_cross_dir = &get_aptcross_dir();
 	$arch = &Cache::Apt::Config::get_cache_architecture();
@@ -426,7 +429,7 @@ updated within the last 24hrs, no update is performed.
 
 =cut
 
-sub check_update()
+sub check_update
 {
 	$verbose = shift;
 	$arch = &Cache::Apt::Config::get_cache_architecture();
@@ -451,7 +454,7 @@ using update_sources.
 
 =cut
 
-sub setup_config()
+sub setup_config
 {
 	$suite = Cache::Apt::Config::get_suite() if (not defined ($suite));
 	$arch = &Cache::Apt::Config::get_cache_architecture();
@@ -550,7 +553,7 @@ cache. Called by setup_config, force_update and check_update.
 
 =cut
 
-sub update_sources()
+sub update_sources
 {
 	$apt_cross_dir = &get_aptcross_dir;
 	$arch = &Cache::Apt::Config::get_cache_architecture;
@@ -625,6 +628,59 @@ sub prepare_sources_list
 	}
 }
 
+=head1 get_provides
+
+Retrieves the ProvidesList for the specified package.
+
+The AptPkg bindings index the ProvidesList under what is
+provided, not what the specified package provides, so this
+routine iterates over the cache and creates a suitable
+index. Subsequent queries are made against the new hash.
+
+e.g. When quering apt-cache, you may expect to see:
+ $ apt-cache show cdebconf | grep Provides
+ Provides: debconf-2.0
+
+However, in the AptPkg bindings, this relationship is
+indexed under debconf-2.0, not under cdebconf. i.e.
+ $cache->{"debconf-2.0"}{ProvidesList}[0]{Name} eq "cdebconf"
+actually returns true when Cache::Apt::Lookup expects:
+ $cache->{"cdebconf"}{ProvidesList}[0]{Name} eq "debconf-2.0"
+to be true.
+
+Returns a comma-separated list of packages that are
+provided by the specified package or undef.
+
+=cut
+
+my $provide_done = 0;
+
+sub get_provides {
+	my $pkg = shift(@_);
+	return $provides{$pkg} if ($provide_done > 0);
+	my $ipkg;
+	my $iter = &get_cache_iter();
+	do {
+		$ipkg = $iter->next;
+		return undef unless (defined $ipkg);
+		if ($apt_cache->{$ipkg}{ProvidesList})
+		{
+			my @list=();
+			my $check;
+			foreach(@{$apt_cache->{$ipkg}{ProvidesList}})
+			{
+				$check = $_->{OwnerPkg}{Name};
+				push @list, $_->{Name};
+			}
+			$provides{$check} = join(', ', @list);
+		}
+		$provide_done++;
+	} while ($ipkg);
+	return $provides{$pkg};
+}
+
+### (End of public functions) ###
+
 sub this_cache { return $apt_cache }
 
 # Set defaults.
@@ -682,6 +738,7 @@ sub get_depends {
 			return;
 		}
 	}
+	&get_provides($pkg);
 	# Which way do our dependencies go?  Reverse, or forward.  Notice
 	# how we get the last version for our forward dependencies.
 	if ($reverse) {
